"""Response hook for pydantic and requests."""

from __future__ import annotations

import typing as t

import requests
from pydantic import BaseModel


class PydanticResponse(requests.Response):
    """An pydantic-enabled :class:`requests.Response <requests.Response>` object.
    Effectively the same, but with an intelligent ``.html`` property added.
    """

    def __init__(self, session: PydanticSession):
        super(PydanticResponse, self).__init__()
        self._object: t.Type[BaseModel] = None
        self.session = session

    @property
    def model(self) -> t.Type[BaseModel]:
        if not self._object:
            try:
                self._object = self.session.handlers[self.status_code](**self.json())
            except ValueError:
                raise ValueError(
                    f"Parser for status code {self.status_code} is missing in session."
                )
        return self._object

    @classmethod
    def _from_response(cls, response, session: PydanticSession):
        html_r = cls(session=session)
        html_r.__dict__.update(response.__dict__)
        return html_r


class PydanticSession(requests.Session):
    """ A consumable session, for cookie persistence and connection pooling,
    amongst other things.
    """

    def __init__(self, handlers: t.Dict[int, t.Type[BaseModel]], headers: dict = None):
        super().__init__()
        self.handlers = handlers
        self.hooks["response"].append(self.response_hook)
        if headers:
            self.headers.update(headers)

    def response_hook(self, response, **kwargs) -> PydanticResponse:
        """ Replace response by PydanticResponse. """
        return PydanticResponse._from_response(response, self)



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['pydantic_requests']

package_data = \
{'': ['*']}

install_requires = \
['pydantic>=0.25.0', 'requests>=2.21']

setup_kwargs = {
    'name': 'pydantic-requests',
    'version': '0.1.2',
    'description': 'A pydantic integration with requests.',
    'long_description': '# Marriage of Pydantic and Requests\n\nA helper that integrates Pydantic with requests library for seamless access to defined Models.\n\n## Example\n\n```python\nfrom enum import Enum\nfrom pydantic import BaseModel\nfrom pydantic_requests import PydanticSession\n\n\nclass DNSStatus(Enum):\n    """DNS OP response codes.\n    ref: https://www.iana.org/assignments/dns-parameters/dns-parameters.xhtml#dns-parameters-6\n    """\n\n    # No Error = 0\n    NoError = 0\n\n    # Format Error = 1\n    FormErr = 1\n\n    # Server Failure\n    ServFail = 2\n\n    # Non-Existent Domain\n    NXDomain = 3\n\n\nclass DNSQuery(BaseModel):\n    Status: DNSStatus\n\n    class Config:\n        """Configure DNS query."""\n\n        allow_mutation = False\n        arbitrary_types_allowed = True\n\n\nwith PydanticSession(\n    {200: DNSQuery}, headers={"accept": "application/dns-json"}\n) as session:\n    domain = "dz0ny.xyz"\n    res = session.get(\n        "https://cloudflare-dns.com/dns-query", params={"name": domain, "type": "NS"}\n    )\n    res.raise_for_status()\n    query: DNSQuery = res.model\n    if query.Status != DNSStatus.NXDomain:\n        print("Domain is registered.")\n    else:\n        print("Domain is registered.")\n\n```',
    'author': 'Janez Troha',
    'author_email': 'dz0ny@ubuntu.si',
    'url': 'https://github.com/dz0ny/pydantic-requests',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
